<?php
/**
 * Created by PhpStorm.
 * User: smp
 * Date: 25/12/18
 * Time: 09:26 AM
 */

namespace Saulmoralespa\PagoFacilChile\Model;

class PagoFacilChile extends \Magento\Payment\Model\Method\AbstractMethod
{
    const CODE = 'pagofacilchile';

    protected $_code = self::CODE;

    protected $_isGateway = true;

    protected $_canOrder = true;

    protected $_canAuthorize = true;

    protected $_canCapture = true;

    protected $_canCapturePartial = true;

    protected $_canRefund = false;

    protected $_canRefundInvoicePartial = false;

    protected $_canVoid = true;

    protected $_canFetchTransactionInfo = true;

    protected $_canReviewPayment = true;

    protected $_supportedCurrencyCodes = array('CLP');

    protected $_minAmount = null;

    protected $_maxAmount = null;

    protected $_tpConnector;

    protected $_pstPagoFacilLogger;


    /**
     * PagoFacilChile constructor.
     * @param \Saulmoralespa\PagoFacilChile\Logger\Logger $pstPagoFacilLogger
     * @param \Magento\Framework\Model\Context $context
     * @param \Magento\Framework\Registry $registry
     * @param \Magento\Framework\Api\ExtensionAttributesFactory $extensionFactory
     * @param \Magento\Framework\Api\AttributeValueFactory $customAttributeFactory
     * @param \Magento\Payment\Helper\Data $paymentData
     * @param \Magento\Framework\App\Config\ScopeConfigInterface $scopeConfig
     * @param \Magento\Payment\Model\Method\Logger $logger
     * @param \Magento\Framework\Model\ResourceModel\AbstractResource|null $resource
     * @param \Magento\Framework\Data\Collection\AbstractDb|null $resourceCollection
     * @param array $data
     */
    public function __construct(
        \Saulmoralespa\PagoFacilChile\Logger\Logger $pstPagoFacilLogger,
        \Saulmoralespa\PagoFacilChile\Model\Factory\Connector $tpc,
        \Magento\Framework\Model\Context $context,
        \Magento\Framework\Registry $registry,
        \Magento\Framework\Api\ExtensionAttributesFactory $extensionFactory,
        \Magento\Framework\Api\AttributeValueFactory $customAttributeFactory,
        \Magento\Payment\Helper\Data $paymentData,
        \Magento\Framework\App\Config\ScopeConfigInterface $scopeConfig,
        \Magento\Payment\Model\Method\Logger $logger,
        \Magento\Framework\Model\ResourceModel\AbstractResource $resource = null,
        \Magento\Framework\Data\Collection\AbstractDb $resourceCollection = null,
        array $data = []
    )
    {
        parent::__construct(
            $context,
            $registry,
            $extensionFactory,
            $customAttributeFactory,
            $paymentData,
            $scopeConfig,
            $logger,
            $resource,
            $resourceCollection,
            $data
        );

        $this->_minAmount = $this->getConfigData('min_order_total');
        $this->_maxAmount = $this->getConfigData('max_order_total');
        $this->_tpConnector = $tpc;
        $this->_pstPagoFacilLogger = $pstPagoFacilLogger;
        $this->_scopeConfig = $scopeConfig;

    }

    /**
     * @param null $storeId
     * @return bool
     */
    public function isActive($storeId = null)
    {
        $pagoFacilChile_active = (bool)(int)$this->getConfigData('active', $storeId);
        if ($pagoFacilChile_active) return true;
        return false;
    }

    /**
     * @param \Magento\Quote\Api\Data\CartInterface|null $quote
     * @return bool
     */
    public function isAvailable(\Magento\Quote\Api\Data\CartInterface $quote = null)
    {

        if ($quote && (
                $quote->getBaseGrandTotal() < $this->_minAmount
                || ($this->_maxAmount && $quote->getBaseGrandTotal() > $this->_maxAmount))
        ) {
            return false;
        }
        if (!$this->_tpConnector->getUserToken() || !$this->_tpConnector->accountId()) {
            return false;
        }
        return parent::isAvailable($quote);
    }

    /**
     * @param string $currencyCode
     * @return bool
     */
    public function canUseForCurrency($currencyCode)
    {
        if (!in_array($currencyCode, $this->_supportedCurrencyCodes)) {
            return false;
        }
        return true;
    }

    public function getOrderStates()
    {
        return array(
            'pending' => $this->_scopeConfig->getValue('payment/pagofacilchile/states/pending', \Magento\Store\Model\ScopeInterface::SCOPE_STORE),
            'approved' => $this->_scopeConfig->getValue('payment/pagofacilchile/states/approved', \Magento\Store\Model\ScopeInterface::SCOPE_STORE),
            'rejected' => $this->_scopeConfig->getValue('payment/pagofacilchile/states/rejected', \Magento\Store\Model\ScopeInterface::SCOPE_STORE)
        );
    }

    public function getAmount($order)
    {
        $amount = $order->getGrandTotal();
        //$currencyCode = $order->getOrderCurrencyCode();
        //if ($currencyCode === 'USD')
        //return number_format($amount, 2, ".", "");
        return round($amount);

    }
}